/**
 * This monster object is used to display messages and ask for user input without blocking the game.
*/

globalvar MapspinMessage;
MapspinMessage = object_add();
object_set_depth(MapspinMessage, -120001);
object_set_parent(MapspinMessage, BubbleMenuZ);
object_event_add(MapspinMessage, ev_other, ev_user0, "// NO");

object_event_add(MapspinMessage, ev_create, 0, "
    STATE_LOGIN = 1;
    STATE_VOTE = 2;
    STATE_ROTATION = 3;
    STATE_MESSAGE = 4;

    msg = '';
    state = 0;
    selected = 0;
    input_string = '';
    keyboard_string = '';
    next = noone;

    // Queue messages
    var obj;
    obj = MapspinMessage.id;
    if (obj == id) {
        alarm[1] = 1;
    }

    while(obj != id) {
        if (obj.next == noone) {
            obj.next = id;
            visible = false;
            break;
        } else if (obj.next.input_string != '') {
            keyboard_string = obj.next.input_string;
        }
        obj = obj.next;
    }
");

object_event_add(MapspinMessage, ev_destroy, 0, "
    // Activate the next message if there is one.
    if (next != noone) {
        next.visible = true;
        keyboard_string = next.input_string;
    }
");

object_event_add(MapspinMessage, ev_alarm, 0, "
    instance_destroy();
");

object_event_add(MapspinMessage, ev_alarm, 1, "
    if (input_string != '') {
        keyboard_string = input_string;
    }
");

object_event_add(MapspinMessage, ev_step, ev_step_normal, "
    if (!visible)
        exit;

    with(BubbleMenuZ)
        done = true;
    with(BubbleMenuX)
        done = true;
    with(BubbleMenuC)
        done = true;

    if (keyboard_check_pressed(vk_escape))
        instance_destroy();

    switch(state) {
        case STATE_LOGIN:
            if (mouse_check_button_pressed(mb_right) || (keyboard_check(vk_control) && keyboard_check_pressed(ord('V'))))
                keyboard_string = clipboard_get_text();
            else if (keyboard_check(vk_control) && keyboard_check_pressed(ord('C'))) {
                if (selected == 0)
                    clipboard_set_text(mapspin.username);
                else
                    clipboard_set_text(mapspin.key);
            }

            if (selected == 0)
                mapspin.username = keyboard_string;
            else
                mapspin.key = keyboard_string;

            if (keyboard_check_pressed(vk_up) || keyboard_check_pressed(vk_down)) {
                selected = !selected;
                if (selected == 0)
                    keyboard_string = mapspin.username;
                else
                    keyboard_string = mapspin.key;
            }
        break;
        case STATE_VOTE:
            if (keyboard_check_pressed(ord('1'))) {
                with(mapspin) {
                    ds_queue_enqueue(downloadQueue, MAPSPIN_BASE_URL + MAPSPIN_VOTE + CustomMapGetMapMD5(global.currentMap) + '/?username=' + username + '&key=' + key + '&score=1');
                    clipboard_set_text(MAPSPIN_BASE_URL + MAPSPIN_VOTE + CustomMapGetMapMD5(global.currentMap) + '/?username=' + username + '&key=' + key + '&score=1')
                    ds_queue_enqueue(actionQueue, ACTION_VOTE);
                }
                instance_destroy();
            } else if (keyboard_check_pressed(ord('2'))) {
                with(mapspin) {
                    ds_queue_enqueue(downloadQueue, MAPSPIN_BASE_URL + MAPSPIN_VOTE + CustomMapGetMapMD5(global.currentMap) + '/?username=' + username + '&key=' + key + '&score=0');
                    ds_queue_enqueue(actionQueue, ACTION_VOTE);
                }
                instance_destroy();
            } else if (keyboard_check_pressed(ord('0'))) {
                instance_destroy();
            }
        break;
        case STATE_ROTATION:
            if (mouse_check_button_pressed(mb_right) || (keyboard_check(vk_control) && keyboard_check_pressed(ord('V'))))
                keyboard_string = clipboard_get_text();
            else if (keyboard_check(vk_control) && keyboard_check_pressed(ord('C')))
                clipboard_set_text(mapspin.rotation);

            mapspin.rotation = keyboard_string;
        break;
        case STATE_MESSAGE:
            if (keyboard_check_pressed(vk_enter)) {
                instance_destroy();
            }
        break;
    }
");

object_event_add(MapspinMessage, ev_draw, 0, "
    var xoffset, yoffset, xsize, ysize, width, height;
    xsize = view_wview[0];
    ysize = view_hview[0];
    xoffset = view_xview[0];
    yoffset = view_yview[0];

    draw_set_valign(fa_left);
    draw_set_halign(fa_top);
    draw_set_color(c_black);
    draw_set_alpha(0.8);

    switch(state) {
        case STATE_LOGIN:
            width = 300;
            height = 100;
            xoffset = round(xoffset+(xsize-width)/2);
            yoffset = round(yoffset+(ysize-height)/2);

            draw_rectangle(xoffset, yoffset, xoffset+width, yoffset+height, false);
            draw_set_alpha(1);
            draw_rectangle(xoffset, yoffset, xoffset+width, yoffset+height, true);
            draw_set_color(c_white);
            draw_text(xoffset+5, yoffset+5, 'Please log in');

            if (selected == 0)
                draw_set_color(c_red);
            else if (mouse_x > xoffset+5 && mouse_x < xoffset+width-5 && mouse_y > yoffset+25 && mouse_y < yoffset+42) {
                if (mouse_check_button_pressed(mb_left)) {
                    selected = 0;
                    keyboard_string = mapspin.username;
                }
                draw_set_color(c_orange);
            } else
                draw_set_color(c_white);

            draw_text(xoffset+5, yoffset+25, 'Mapspin username');
            draw_rectangle(xoffset+140, yoffset+23, xoffset+width-5, yoffset+40, true);
            draw_text(xoffset+142, yoffset+25, mapspin.username);

            if (selected == 1)
                draw_set_color(c_red);
            else if (mouse_x > xoffset+5 && mouse_x < xoffset+width-5 && mouse_y > yoffset+45 && mouse_y < yoffset+62) {
                if (mouse_check_button_pressed(mb_left)) {
                    selected = 1;
                    keyboard_string = mapspin.key;
                }
                draw_set_color(c_orange);
            } else
                draw_set_color(c_white);

            draw_text(xoffset+5, yoffset+45, 'Mapspin key');
            draw_rectangle(xoffset+140, yoffset+43, xoffset+width-5, yoffset+60, true);
            draw_text(xoffset+142, yoffset+45, mapspin.key);

            draw_set_color(c_gray);
            draw_rectangle(xoffset+5, yoffset+height-25, xoffset+55, yoffset+height-5, false);
            draw_rectangle(xoffset+65, yoffset+height-25, xoffset+135, yoffset+height-5, false);
            draw_rectangle(xoffset+145, yoffset+height-25, xoffset+205, yoffset+height-5, false);

            if (mouse_x > xoffset+5 && mouse_x < xoffset+55 && mouse_y > yoffset+height-25 && mouse_y < yoffset+height-5) {
                if (mouse_check_button_pressed(mb_left)) {
                    with(mapspin) {
                        username = string_replace_all(string_replace_all(username, ' ', '+'), chr(43), '%27');
                        key = string_replace_all(string_replace_all(key, ' ', '+'), chr(43), '%27');

                        ds_queue_enqueue(downloadQueue, MAPSPIN_BASE_URL + MAPSPIN_LOGIN + '?username=' + username + '&key=' + key);
                        ds_queue_enqueue(actionQueue, ACTION_LOGIN);
                    }
                    instance_destroy();
                }
                draw_set_color(c_red);
            } else
                draw_set_color(c_white);
            draw_rectangle(xoffset+5, yoffset+height-25, xoffset+55, yoffset+height-5, true);
            draw_text(xoffset+7, yoffset+height-19, 'Login');

            if (mouse_x > xoffset+65 && mouse_x < xoffset+135 && mouse_y > yoffset+height-25 && mouse_y < yoffset+height-5) {
                if (mouse_check_button_pressed(mb_left)) {
                    with(mapspin) {
                        if (string_length(mapspin.username) <= 3)
                            mapspin.username = global.playerName;
                        while(string_length(mapspin.username) <= 3)
                            mapspin.username += '_';

                        username = string_replace_all(string_replace_all(username, ' ', '+'), chr(43), '%27');

                        ds_queue_enqueue(downloadQueue, MAPSPIN_BASE_URL + MAPSPIN_REGISTER + '?username=' + username);
                        ds_queue_enqueue(actionQueue, ACTION_REGISTER);
                    }
                    instance_destroy();
                }
                draw_set_color(c_red);
            } else
                draw_set_color(c_white);
            draw_rectangle(xoffset+65, yoffset+height-25, xoffset+135, yoffset+height-5, true);
            draw_text(xoffset+67, yoffset+height-19, 'Register');

            if (mouse_x > xoffset+145 && mouse_x < xoffset+205 && mouse_y > yoffset+height-25 && mouse_y < yoffset+height-5) {
                if (mouse_check_button_pressed(mb_left)) {
                    tryLogin = false;
                    ini_open('gg2.ini');
                    ini_write_real('Settings', 'Mapspin trylogin', tryLogin);
                    ini_close();
                    instance_destroy();
                }
                draw_set_color(c_red);
            } else
                draw_set_color(c_white);
            draw_rectangle(xoffset+145, yoffset+height-25, xoffset+205, yoffset+height-5, true);
            draw_text(xoffset+147, yoffset+height-19, 'Cancel');
        break;
        case STATE_VOTE:
            width = 100;
            height = 80;
            yoffset = round(yoffset+(ysize-height)/2);

            draw_rectangle(xoffset, yoffset, xoffset+width, yoffset+height, false);
            draw_set_alpha(1);
            draw_rectangle(xoffset, yoffset, xoffset+width, yoffset+height, true);
            draw_set_color(c_white);
            draw_text(xoffset+5, yoffset+5, 'Map voting');
            draw_text(xoffset+5, yoffset+25, '1. Like');
            draw_text(xoffset+5, yoffset+42, '2. Dislike');
            draw_text(xoffset+5, yoffset+59, '0. Cancel');
        break;
        case STATE_ROTATION:
            width = 300;
            height = 80;
            xoffset = round(xoffset+(xsize-width)/2);
            yoffset = round(yoffset+(ysize-height)/2);

            draw_rectangle(xoffset, yoffset, xoffset+width, yoffset+height, false);
            draw_set_alpha(1);
            draw_rectangle(xoffset, yoffset, xoffset+width, yoffset+height, true);
            draw_set_color(c_white);
            draw_text(xoffset+5, yoffset+5, 'Select Maprotation');

            draw_set_color(c_red);
            draw_text(xoffset+5, yoffset+25, 'Maprotation name');
            draw_rectangle(xoffset+140, yoffset+23, xoffset+width-5, yoffset+40, true);
            draw_text(xoffset+142, yoffset+25, mapspin.rotation);

            draw_set_color(c_gray);
            draw_rectangle(xoffset+5, yoffset+height-25, xoffset+75, yoffset+height-5, false);
            draw_rectangle(xoffset+85, yoffset+height-25, xoffset+145, yoffset+height-5, false);

            if (mouse_x > xoffset+5 && mouse_x < xoffset+75 && mouse_y > yoffset+height-25 && mouse_y < yoffset+height-5) {
                if (mouse_check_button_pressed(mb_left)) {
                    with(mapspin) {
                        rotation = string_replace_all(string_replace_all(rotation, ' ', '%20'), chr(43), '%27');
                        ds_queue_enqueue(downloadQueue, MAPSPIN_BASE_URL + MAPSPIN_GENERATE + rotation + '/');
                        ds_queue_enqueue(actionQueue, ACTION_DL_ROTATION);
                    }
                    instance_destroy();
                }
                draw_set_color(c_red);
            } else
                draw_set_color(c_white);
            draw_rectangle(xoffset+5, yoffset+height-25, xoffset+75, yoffset+height-5, true);
            draw_text(xoffset+7, yoffset+height-19, 'Download');

            if (mouse_x > xoffset+85 && mouse_x < xoffset+145 && mouse_y > yoffset+height-25 && mouse_y < yoffset+height-5) {
                if (mouse_check_button_pressed(mb_left))
                    instance_destroy();
                draw_set_color(c_red);
            } else
                draw_set_color(c_white);
            draw_rectangle(xoffset+85, yoffset+height-25, xoffset+145, yoffset+height-5, true);
            draw_text(xoffset+87, yoffset+height-19, 'Cancel');
        break;
        case STATE_MESSAGE:
            width = 10 + string_width(msg);
            height = 10 + 25 + string_height(msg);
            xoffset = round(xoffset+(xsize-width)/2);
            yoffset = round(yoffset+(ysize-height)/2);

            draw_rectangle(xoffset, yoffset, xoffset+width, yoffset+height, false);
            draw_set_alpha(1);
            draw_rectangle(xoffset, yoffset, xoffset+width, yoffset+height, true);
            draw_set_color(c_white);
            draw_text(xoffset+5, yoffset+5, msg);

            draw_set_color(c_gray);
            draw_rectangle(xoffset+5, yoffset+height-25, xoffset+35, yoffset+height-5, false);

            if (mouse_x > xoffset+5 && mouse_x < xoffset+35 && mouse_y > yoffset+height-25 && mouse_y < yoffset+height-5) {
                if (mouse_check_button_pressed(mb_left))
                    instance_destroy();
                draw_set_color(c_red);
            } else
                draw_set_color(c_white);
            draw_rectangle(xoffset+5, yoffset+height-25, xoffset+35, yoffset+height-5, true);
            draw_text(xoffset+7, yoffset+height-19, 'OK');
        break;
        default:
            instance_destroy();
        break;
    }
");
